# Generated by Django 2.2.9 on 2020-01-01 16:58

from django.db import migrations
import wagtail_links.fields


class Migration(migrations.Migration):

    dependencies = [
        ("wagtail_links", "0006_auto_20200101_1646"),
    ]

    operations = [
        migrations.AlterField(
            model_name="link",
            name="name",
            field=wagtail_links.fields.NullSlugField(
                help_text="Unique name for this link (for use by Django templates).",
                unique=True,
                verbose_name="Name",
            ),
        ),
    ]



from django import template
from ..models import Link
import logging


logger = logging.getLogger(__name__)

register = template.Library()


@register.simple_tag
def get_wagtail_link(link_name):
    try:
        link = Link.objects.get_by_natural_key(link_name)
    except Link.DoesNotExist:
        logger.error("Wagtail Link with name [%s] does not exist.", link_name)
        return None
    return link


@register.simple_tag
def get_wagtail_link_url(link_name):
    link = get_wagtail_link(link_name)
    if link is None:
        return ""
    return link.url



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-22 20:15
from __future__ import unicode_literals

from django.db import migrations, models
import wagtail_links.models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtail_links", "0002_auto_20170522_1552"),
    ]

    operations = [
        migrations.AlterField(
            model_name="link",
            name="django_view_name",
            field=models.CharField(
                blank=True,
                help_text="Name of django view for reverse lookup",
                max_length=255,
                validators=[wagtail_links.models.validate_django_reverse],
            ),
        ),
    ]



from django.core.exceptions import ValidationError
from django.template import Context, Template
from wagtail.test.utils import WagtailPageTests
from wagtail.models import Page
from unittest.mock import patch
from .models import Link


class WagtailLinksTest(WagtailPageTests):
    def test_get_link(self):
        link = Link.objects.create(link_external="https://example.com")
        self.assertEqual(link.url, "https://example.com")

        link = Link.objects.create(link_relative="/example.com")
        self.assertEqual(link.url, "/example.com")

        page = Page.objects.first()
        link = Link.objects.create(link_page=page)
        self.assertEqual(link.url, "")

        link = Link.objects.create(django_view_name="admin:index")
        self.assertEqual(link.url, "/admin/")

    def test_link_validation(self):
        link = Link(
            link_external="https://example.com",
            link_relative="/foo",
        )
        with self.assertRaises(ValidationError):
            link.clean()

        link = Link()
        with self.assertRaises(ValidationError):
            link.clean()

        link = Link(django_view_name="doesnotexist")
        with self.assertRaises(ValidationError):
            link.full_clean()

    @patch("wagtail_links.models.logger")
    def test_log_broken_links(self, mock_logger):
        link = Link.objects.create(django_view_name="doesnotexist")
        link.url
        mock_logger.warning.assert_called_with(
            "Unable to reverse Django URL for Link[id=%s]", link.pk
        )

    def test_cast_link_to_string(self):
        link = Link.objects.create(name="example", link_external="https://example.com")
        self.assertEqual("{}".format(link), "Link[name=example]: https://example.com")

        link = Link.objects.create(link_external="https://example.com")
        self.assertEqual(
            "{}".format(link),
            "Link[link_external=https://example.com]: https://example.com",
        )

        link = Link.objects.create(link_relative="/example.com")
        self.assertEqual(
            "{}".format(link), "Link[link_relative=/example.com]: /example.com"
        )

        page = Page.objects.first()
        link = Link.objects.create(link_page=page)
        self.assertEqual("{}".format(link), "Link[link_page=Root]: No Link URL")

        link = Link.objects.create(django_view_name="admin:index")
        self.assertEqual(
            "{}".format(link), "Link[django_view_name=admin:index]: /admin/"
        )

    def test_multiple_blank_link_names(self):
        link1 = Link.objects.create(name="", link_external="https://example.com/1")
        link2 = Link.objects.create(name="", link_external="https://example.com/2")
        self.assertEqual(link1.name, "")
        self.assertEqual(link2.name, "")


class WagtailLinksTagsTest(WagtailPageTests):
    def test_get_wagtail_link(self):
        Link.objects.create(name="example", link_external="https://example.com")
        context = Context({})
        template = Template(
            """
            {% load wagtail_links %}
            {% get_wagtail_link 'example' as my_link %}
            <a href="{{ my_link.url }}">Testing</a>
            """
        )
        rendered_template = template.render(context)
        self.assertInHTML(
            '<a href="https://example.com">Testing</a>', rendered_template
        )

    def test_get_wagtail_link_url(self):
        Link.objects.create(name="example", link_external="https://example.com/test2/")
        context = Context({})
        template = Template(
            """
            {% load wagtail_links %}
            <a href="{% get_wagtail_link_url 'example' %}">Testing</a>
            """
        )
        rendered_template = template.render(context)
        self.assertInHTML(
            '<a href="https://example.com/test2/">Testing</a>', rendered_template
        )



from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.db import models
from django.utils.translation import gettext_lazy as _
from django.urls import reverse, NoReverseMatch
from wagtail.admin.panels import PageChooserPanel, FieldPanel
from wagtail.search import index
from wagtail.snippets.models import register_snippet
from .fields import NullSlugField
import logging

logger = logging.getLogger(__name__)


def validate_django_reverse(view_name):
    try:
        reverse(view_name)
    except NoReverseMatch:
        # Translators: CMS error message
        raise ValidationError(_("Invalid Django view name"))


class LinkManager(models.Manager):
    def get_by_natural_key(self, name):
        return self.get(name=name)


@register_snippet
class Link(index.Indexed, models.Model):
    """
    A generic link that points somewhere else using various methods
    """

    title = models.CharField(
        _("Title"),
        max_length=200,
        blank=True,
        help_text=_("Description of link for use in ARIA compliance"),
    )
    name = NullSlugField(
        _("Name"),
        unique=True,
        help_text=_("Unique name for this link (for use by Django templates)."),
    )
    link_external = models.URLField(
        _("External Link"), blank=True, help_text=_("Absolute URL Link")
    )
    link_relative = models.CharField(
        _("Relative Link"),
        max_length=200,
        blank=True,
        validators=[
            RegexValidator(
                regex=r"^(?!www\.|(?:http|ftp)s?://|[A-Za-z]:\\|//).*",
                message="Not valid relative url",
            )
        ],
        help_text=_("Relative URL Link"),
    )
    link_page = models.ForeignKey(
        "wagtailcore.Page",
        verbose_name=_("Wagtail Page Link"),
        null=True,
        blank=True,
        related_name="+",
        on_delete=models.CASCADE,
        help_text=_("Wagtail Page Link"),
    )
    django_view_name = models.CharField(
        _("Django View Link"),
        max_length=255,
        blank=True,
        validators=[validate_django_reverse],
        help_text=_("Name of Django view for reverse lookup"),
    )

    objects = LinkManager()

    panels = [
        FieldPanel("title"),
        FieldPanel("name"),
        FieldPanel("link_external"),
        FieldPanel("link_relative"),
        PageChooserPanel("link_page"),
        FieldPanel("django_view_name"),
    ]

    search_fields = [
        index.RelatedFields(
            "link_page", [index.SearchField("title", partial_match=True)]
        ),
        index.SearchField("title", partial_match=True),
    ]

    class Meta:
        # Translators: Internal Model Name (singular)
        verbose_name = _("Link")
        # Translators: Internal Model Name (plural)
        verbose_name_plural = _("Links")

    def natural_key(self):
        return (self.name,)

    def __str__(self):
        url = self.url
        if not url:
            url = _("No Link URL")
        ctx = {
            "name": self.name,
            "link_external": self.link_external,
            "link_relative": self.link_relative,
            "link_page": self.link_page,
            "django_view_name": self.django_view_name,
            "url": url,
        }
        if self.name:
            return _("Link[name=%(name)s]: %(url)s") % ctx
        if self.link_external:
            return _("Link[link_external=%(link_external)s]: %(url)s") % ctx
        if self.link_relative:
            return _("Link[link_relative=%(link_relative)s]: %(url)s") % ctx
        if self.link_page:
            return _("Link[link_page=%(link_page)s]: %(url)s") % ctx
        return _("Link[django_view_name=%(django_view_name)s]: %(url)s") % ctx

    @property
    def url(self):
        """Get URL for use in template"""
        # 1. External Links
        if self.link_external:
            return self.link_external
        # 2. Relative Links
        if self.link_relative:
            return self.link_relative
        # 3. Wagtail Page Links
        if self.link_page:
            url = self.link_page.url
            if url is None:
                return ""
            return url
        # 4. Django View Links
        try:
            return reverse(self.django_view_name)
        except NoReverseMatch:
            logger.warning("Unable to reverse Django URL for Link[id=%s]", self.id)
        # 5. Error Fallback
        return ""

    def clean(self):
        # Don't allow multiple link types to be used
        number_used = (
            bool(self.link_external)
            + bool(self.link_relative)
            + bool(self.link_page)
            + bool(self.django_view_name)
        )
        if number_used > 1:
            raise ValidationError(_("You may only use one link type"))
        if number_used == 0:
            raise ValidationError(_("You must use exactly one link type"))



#!/usr/bin/env python
from setuptools import setup, find_packages, Distribution
import codecs
import os.path

# Make sure versiontag exists before going any further
Distribution().fetch_build_eggs("versiontag>=1.2.0")

from versiontag import get_version, cache_git_tag  # NOQA


packages = find_packages("src")

install_requires = [
    "wagtail>=3.0,<5.1",
]

extras_require = {
    "development": [
        "flake8>=3.3.0",
        "tox>=2.7.0",
        "versiontag>=1.2.0",
    ],
}


def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)


def read(fname):
    return codecs.open(fpath(fname), encoding="utf-8").read()


cache_git_tag()

setup(
    name="wagtail-links",
    description="Wagtail links provides a consistent way to refer to links in a wagtail page.",
    version=get_version(pypi=True),
    long_description=open("README.rst").read(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Wagtail",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
    ],
    keywords="django wagtail",
    author="thelabnyc",
    author_email="thelabdev@thelabnyc.com",
    url="https://gitlab.com/thelabnyc/wagtail-links",
    license="ISC",
    package_dir={"": "src"},
    packages=packages,
    include_package_data=True,
    install_requires=install_requires,
    extras_require=extras_require,
)



# Generated by Django 2.2.9 on 2020-01-01 16:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtail_links", "0005_auto_20191203_1908_squashed_0006_auto_20191204_2129"),
    ]

    operations = [
        migrations.AlterField(
            model_name="link",
            name="title",
            field=models.CharField(
                blank=True,
                help_text="Description of link for use in ARIA compliance",
                max_length=200,
                verbose_name="Title",
            ),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class WagtailLinksConfig(AppConfig):
    name = "wagtail_links"
    # Translators: Backend Library Name
    verbose_name = _("Wagtail Links")



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-22 15:04
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("wagtailcore", "0033_remove_golive_expiry_help_text"),
    ]

    operations = [
        migrations.CreateModel(
            name="Link",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "link_external",
                    models.URLField(blank=True, verbose_name="External link"),
                ),
                (
                    "link_relative",
                    models.URLField(blank=True, verbose_name="External link"),
                ),
                (
                    "django_view_name",
                    models.CharField(
                        blank=True,
                        help_text="Name of django view for reverse lookup",
                        max_length=255,
                    ),
                ),
                (
                    "link_page",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="wagtailcore.Page",
                    ),
                ),
            ],
        ),
    ]
